@@ -274,12 +274,10 @@ void resolve_symbols(Context<E> &ctx) {
     // Redo name resolution from scratch.
     tbb::parallel_for_each(ctx.objs, [&](ObjectFile<E> *file) {
       file->clear_symbols();
-      file->is_alive = !file->is_in_lib;
     });
 
     tbb::parallel_for_each(ctx.dsos, [&](SharedFile<E> *file) {
       file->clear_symbols();
-      file->is_alive = !file->is_needed;
     });
 
     // Remove IR object files.
@@ -293,6 +291,29 @@ void resolve_symbols(Context<E> &ctx) {
   }
 }
 
+// .eh_frame sections are parsed and regenerated by the linker for the purpose
+// of deduplication and garbage collection. As such, the input sections should
+// not be copied over.
+// However, in very rare cases (e.g. GCC CRT compiled with LTO) we might need
+// to resolve cross-object .eh_frame section references (they only point to
+// begin or end and don't depend on the actual section contents).
+// Therefore, the sections are "killed" after symbol resolution as a separate
+// pass.
+template <typename E>
+void kill_eh_frame_sections(Context<E> &ctx) {
+  Timer t(ctx, "kill_eh_frame_sections");
+
+  tbb::parallel_for_each(ctx.objs, [&](ObjectFile<E> *file) {
+    for (i64 i = 0; i < file->sections.size(); i++) {
+      if (std::unique_ptr<InputSection<E>> &isec = file->sections[i]) {
+        if (isec && isec->is_alive && isec->name() == ".eh_frame") {
+          isec->is_alive = false;
+        }
+      }
+    }
+  });
+}
+
 template <typename E>
 void resolve_section_pieces(Context<E> &ctx) {
   Timer t(ctx, "resolve_section_pieces");
@@ -636,7 +657,7 @@ get_start_stop_name(Context<E> &ctx, Chunk<E> &chunk) {
     if (ctx.arg.start_stop) {
       auto isalnum = [](char c) {
         return ('a' <= c && c <= 'z') || ('A' <= c && c <= 'Z') ||
-          ('0' <= c && c <= '9');
+               ('0' <= c && c <= '9');
       };
 
       std::string s{chunk.name};
@@ -773,19 +794,20 @@ void add_synthetic_symbols(Context<E> &ctx) {
 template <typename E>
 void check_cet_errors(Context<E> &ctx) {
   bool warning = (ctx.arg.z_cet_report == CET_REPORT_WARNING);
-  assert(warning || (ctx.arg.z_cet_report == CET_REPORT_ERROR));
+  assert(warning || ctx.arg.z_cet_report == CET_REPORT_ERROR);
 
   auto has_feature = [](ObjectFile<E> *file, u32 feature) {
     return std::any_of(file->gnu_properties.begin(), file->gnu_properties.end(),
-                       [&](auto kv) {
-                         return kv.first == GNU_PROPERTY_X86_FEATURE_1_AND
-                             && (kv.second & feature);
-                       });
+                       [&](std::pair<u32, u32> kv) {
+      return kv.first == GNU_PROPERTY_X86_FEATURE_1_AND &&
+             (kv.second & feature);
+    });
   };
 
   for (ObjectFile<E> *file : ctx.objs) {
     if (file == ctx.internal_obj)
       continue;
+
     if (!has_feature(file, GNU_PROPERTY_X86_FEATURE_1_IBT)) {
       if (warning)
         Warn(ctx) << *file << ": -cet-report=warning: "
@@ -2592,6 +2614,7 @@ template void create_internal_file(Context<E> &);
 template void apply_exclude_libs(Context<E> &);
 template void create_synthetic_sections(Context<E> &);
 template void resolve_symbols(Context<E> &);
+template void kill_eh_frame_sections(Context<E> &);
 template void resolve_section_pieces(Context<E> &);
 template void convert_common_symbols(Context<E> &);
 template void compute_merged_section_sizes(Context<E> &);
